#include "Body.h"
#include <cmath>

Body::Body(const V2f &end1, const V2f &end2, float mass)
    : M(mass), oneOverM( 1.0f / mass ),
      lengthOverTwo((end1 - end2).magnitude() * 0.5f),
      I(mass * lengthOverTwo * lengthOverTwo / 3.0f), oneOverI(1.0f / I),
      _end1orig(end1), _end2orig(end2)
{
    reset();
}

void
Body::reset()
{
    pos = (_end1orig + _end2orig) * 0.5f;
    vel = 0.0f;
    F   = 0.0f;

    V2f diff = _end1orig - _end2orig;
    theta    = (float) atan2(diff.y, diff.x);
    omega    = 0;
    N        = 0;

    //
    // I could also normalize diff... whatever
    //
    dir      = V2f((float) cos(theta), (float) sin(theta));
    end1pos  = _end1orig;
    end2pos  = _end2orig;
}

void
Body::unparse(ostream &out) const
{
    out << "(body " << _end1orig << " " << _end2orig << " " << M << ")";
}
