#include "Exp.h"

#include <cmath>
#include <cstdlib>

//----------------------------------------------------------------------------

Exp::Exp()
{
    // nothing
}

Exp::~Exp()
{
    // nothing
}

void
Exp::mutate(float prob)
{
    // nothing
}

bool
Exp::mutationOccured(float prob)
{
    float rand_val = rand() / (float) RAND_MAX;

    if (rand_val < prob)
    {
        return true;
    }

    return false;
}

Exp *
Exp::getRandomNode(int max_depth)
{
    Exp *result = 0;

    if (max_depth <= 0)
    {
        switch (rand() % 2)
        {
          case 0:
            result = new ConstExp((float) rand() / (float) RAND_MAX - 0.5f);
            break;

          case 1:
            result = new TimeExp();
            break;
        }
    }
    else
    {
        switch (rand() % 9)
        {
          case 0:
            result = new ConstExp((float) rand() / (float) RAND_MAX - 0.5f);
            break;

          case 1:
            result = new TimeExp();
            break;

          case 2:
            result = new UnaryMinusExp(getRandomNode(max_depth - 1));
            break;

          case 3:
            result = new SinExp(getRandomNode(max_depth - 1));
            break;

          case 4:
            result = new CosExp(getRandomNode(max_depth - 1));
            break;

          case 5:
            result = new PlusExp(getRandomNode(max_depth - 1),
                                 getRandomNode(max_depth - 1));
            break;

          case 6:
            result = new TimesExp(getRandomNode(max_depth - 1),
                                  getRandomNode(max_depth - 1));
            break;
          case 7:
            result = new GreaterExp(new ConstExp(rand() / (float) RAND_MAX - 0.5f),
                                    getRandomNode(max_depth - 1),
                                    getRandomNode(max_depth - 1));
            break;
          case 8:
            result = new LessExp(new ConstExp(rand() / (float) RAND_MAX - 0.5f),
                                 getRandomNode(max_depth - 1),
                                 getRandomNode(max_depth - 1));
            break;
        }
    }

    return result;
}

Exp *
Exp::getRandomNodeZero(int max_depth)
{
    Exp *result = 0;

    if (max_depth <= 0)
    {
        switch (rand() % 2)
        {
          case 0:
            result = new ConstExp(0.0f);
            break;

          case 1:
            result = new TimeExp();
            break;
        }
    }
    else
    {
        switch (rand() % 5)
        {
          case 0:
            result = new ConstExp(0.0f);
            break;

          case 1:
            result = new TimeExp();
            break;

          case 2:
            result = new UnaryMinusExp(getRandomNodeZero(max_depth - 1));
            break;

          case 3:
            result = new SinExp(getRandomNode(max_depth - 1));
            break;

          case 4:
            result = new TimesExp(getRandomNodeZero(max_depth - 1),
                                  getRandomNode(max_depth - 1));
            break;
        }
    }

    return result;
}

//----------------------------------------------------------------------------

LeafExp::LeafExp()
{
    // nothing
}

LeafExp::~LeafExp()
{
    // nothing
}

int
LeafExp::getNumInputs() const
{
    return 0;
}

void
LeafExp::mutate(float prob)
{
    Exp::mutate(prob);
}

//----------------------------------------------------------------------------

ConstExp::ConstExp(float val)
    : _val(val)
{
    // nothing
}

ConstExp::~ConstExp()
{
    // nothing
}

float
ConstExp::evaluate(float) const
{
    return _val;
}

float
ConstExp::devaluate(float) const
{
    return 0.0f;
}

void
ConstExp::mutate(float prob)
{
    LeafExp::mutate(prob);

    float rand_val = rand() / (float) RAND_MAX;

    if (rand_val < prob)
    {
        _val = _val * (prob - 2.0f * rand_val);
    }
}

void
ConstExp::unparse(ostream &out) const
{
    out << "(" << _val << ")";
}

ConstExp *
ConstExp::duplicate() const
{
    return new ConstExp(_val);
}

//----------------------------------------------------------------------------

TimeExp::TimeExp()
{
    // nothing
}

TimeExp::~TimeExp()
{
    // nothing
}

float
TimeExp::evaluate(float t) const
{
    return t;
}

float
TimeExp::devaluate(float t) const
{
    return t;
}

void
TimeExp::mutate(float prob)
{
    LeafExp::mutate(prob);
}

void
TimeExp::unparse(ostream &out) const
{
    out << "(t)";
}

TimeExp *
TimeExp::duplicate() const
{
    return new TimeExp();
}

//----------------------------------------------------------------------------

UnaryExp::UnaryExp(Exp *child)
    : _child(child)
{
    if (child == 0)
    {
        throw ExpExc("<UnaryExp::UnaryExp> child was null");
    }
}

UnaryExp::~UnaryExp()
{
    delete _child;
}

int
UnaryExp::getNumInputs() const
{
    return 1;
}

void
UnaryExp::mutate(float prob)
{
    Exp::mutate(prob);

    if (mutationOccured(prob))
    {
        delete _child;
        _child = getRandomNode(rand() % 3 + 1);
    }

    _child->mutate(prob);
}

//----------------------------------------------------------------------------

UnaryMinusExp::UnaryMinusExp(Exp *child)
    : UnaryExp(child)
{
    // nothing
}

UnaryMinusExp::~UnaryMinusExp()
{
    // nothing
}

float
UnaryMinusExp::evaluate(float t) const
{
    return -(_child->evaluate(t));
}

float
UnaryMinusExp::devaluate(float t) const
{
    return -(_child->devaluate(t));
}

void
UnaryMinusExp::mutate(float prob)
{
    UnaryExp::mutate(prob);
}

void
UnaryMinusExp::unparse(ostream &out) const
{
    out << "(- ";
    _child->unparse(out);
    out << ")";
}

UnaryMinusExp *
UnaryMinusExp::duplicate() const
{
    return new UnaryMinusExp(_child->duplicate());
}

//----------------------------------------------------------------------------

SinExp::SinExp(Exp *child)
    : UnaryExp(child)
{
    // nothing
}

SinExp::~SinExp()
{
    // nothing
}

float
SinExp::evaluate(float t) const
{
    return sinf(_child->evaluate(t));
}

float
SinExp::devaluate(float t) const
{
    return cosf(_child->evaluate(t)) * _child->devaluate(t);
}

void
SinExp::mutate(float prob)
{
    UnaryExp::mutate(prob);
}

void
SinExp::unparse(ostream &out) const
{
    out << "(sin ";
    _child->unparse(out);
    out << ")";
}

SinExp *
SinExp::duplicate() const
{
    return new SinExp(_child->duplicate());
}

//----------------------------------------------------------------------------

CosExp::CosExp(Exp *child)
    : UnaryExp(child)
{
    // nothing
}

CosExp::~CosExp()
{
    // nothing
}

float
CosExp::evaluate(float t) const
{
    return cosf(_child->evaluate(t));
}

float
CosExp::devaluate(float t) const
{
    return -sinf(_child->evaluate(t)) * _child->devaluate(t);
}

void
CosExp::mutate(float prob)
{
    UnaryExp::mutate(prob);
}

void
CosExp::unparse(ostream &out) const
{
    out << "(cos ";
    _child->unparse(out);
    out << ")";
}

CosExp *
CosExp::duplicate() const
{
    return new CosExp(_child->duplicate());
}

//----------------------------------------------------------------------------

BinaryExp::BinaryExp(Exp *lchild, Exp *rchild)
    : _lchild(lchild), _rchild(rchild)
{
    if (lchild == 0 || rchild == 0)
    {
        throw ExpExc("<BinaryExp::BinaryExp> a child was null");
    }
}

BinaryExp::~BinaryExp()
{
    delete _lchild;
    delete _rchild;
}

int 
BinaryExp::getNumInputs() const
{
    return 2;
}

void
BinaryExp::mutate(float prob)
{
    Exp::mutate(prob);

    if (mutationOccured(prob))
    {
        switch (rand() % 3)
        {
          case 0:
            delete _lchild;
            _lchild = getRandomNode(rand() % 3 + 1);
            break;

          case 1:
            delete _rchild;
            _rchild = getRandomNode(rand() % 3 + 1);
            break;

          case 2:
            Exp *tmp = _lchild;
            _lchild  = _rchild;
            _rchild  = tmp;
            break;
        }
    }

    _lchild->mutate(prob);
    _rchild->mutate(prob);
}

//----------------------------------------------------------------------------

PlusExp::PlusExp(Exp *lchild, Exp *rchild)
    : BinaryExp(lchild, rchild)
{
    // nothing
}

PlusExp::~PlusExp()
{
    // nothing
}

float
PlusExp::evaluate(float t) const
{
    return _lchild->evaluate(t) + _rchild->evaluate(t);
}

float
PlusExp::devaluate(float t) const
{
    return _lchild->devaluate(t) + _rchild->devaluate(t);
}

void
PlusExp::mutate(float prob)
{
    BinaryExp::mutate(prob);
}

void
PlusExp::unparse(ostream &out) const
{
    out << "(+ ";
    _lchild->unparse(out);
    out << " ";
    _rchild->unparse(out);
    out << ")";
}

PlusExp *
PlusExp::duplicate() const
{
    return new PlusExp(_lchild->duplicate(), _rchild->duplicate());
}

//----------------------------------------------------------------------------

TimesExp::TimesExp(Exp *lchild, Exp *rchild)
    : BinaryExp(lchild, rchild)
{
    // nothing
}

TimesExp::~TimesExp()
{
    // nothing
}

float
TimesExp::evaluate(float t) const
{
    return _lchild->evaluate(t) * _rchild->evaluate(t);
}

float
TimesExp::devaluate(float t) const
{
    return _lchild->evaluate(t) * _rchild->devaluate(t)
        + _lchild->devaluate(t) * _rchild->evaluate(t);
}

void
TimesExp::mutate(float prob)
{
    BinaryExp::mutate(prob);
}

void
TimesExp::unparse(ostream &out) const
{
    out << "(* ";
    _lchild->unparse(out);
    out << " ";
    _rchild->unparse(out);
    out << ")";
}

TimesExp *
TimesExp::duplicate() const
{
    return new TimesExp(_lchild->duplicate(), _rchild->duplicate());
}

//----------------------------------------------------------------------------

GreaterExp::GreaterExp(ConstExp *val, Exp *lchild, Exp *rchild)
    : BinaryExp(lchild, rchild), _val(val)
{
    // nothing
}

GreaterExp::~GreaterExp()
{
    // nothing
}

float
GreaterExp::evaluate(float t) const
{
    if (t > _val->evaluate(t))
        return _lchild->evaluate(t);

    return _rchild->evaluate(t);
}

float
GreaterExp::devaluate(float t) const
{
    if (t > _val->evaluate(t))
        return _lchild->devaluate(t);

    return _rchild->devaluate(t);
}

void
GreaterExp::mutate(float prob)
{
    BinaryExp::mutate(prob);
    _val->mutate(prob);
}

void
GreaterExp::unparse(ostream &out) const
{
    out << "(> ";
    _val->unparse(out);
    out << " ";
    _lchild->unparse(out);
    out << " ";
    _rchild->unparse(out);
    out << ")";
}

GreaterExp *
GreaterExp::duplicate() const
{
    return new GreaterExp(_val->duplicate(), _lchild->duplicate(), _rchild->duplicate());
}

//----------------------------------------------------------------------------

LessExp::LessExp(ConstExp *val, Exp *lchild, Exp *rchild)
    : BinaryExp(lchild, rchild), _val(val)
{
    // nothing
}

LessExp::~LessExp()
{
    // nothing
}

float
LessExp::evaluate(float t) const
{
    if (t < _val->evaluate(t))
        return _lchild->evaluate(t);

    return _rchild->evaluate(t);
}

float
LessExp::devaluate(float t) const
{
    if (t < _val->evaluate(t))
        return _lchild->devaluate(t);

    return _rchild->devaluate(t);
}

void
LessExp::mutate(float prob)
{
    BinaryExp::mutate(prob);
    _val->mutate(prob);
}

void
LessExp::unparse(ostream &out) const
{
    out << "(< ";
    _val->unparse(out);
    out << " ";
    _lchild->unparse(out);
    out << " ";
    _rchild->unparse(out);
    out << ")";
}

LessExp *
LessExp::duplicate() const
{
    return new LessExp(_val->duplicate(), _lchild->duplicate(), _rchild->duplicate());
}

