#include "Genetic.h"

#include <cstdlib>
#include <iostream>

int
main(int argc, char *argv[])
{
    GenePool pool;

    if (argc != 6)
    {
        cout << "Usage: " << argv[0] 
             << " <input file> <output file> <num_gen> <max_pop> <sim_len>" << endl;
        return -1;
    }


    int error = pool.parse(argv[1]);

    if (error != 0)
    {
        cerr << "Parse error, exiting" << endl;
        return error;
    }

    int num_gen = atoi(argv[3]);

    srand(num_gen * 30342 - 338);

    pool.setMaxPopulation(atoi(argv[4]));
    pool.setSimulationLength(atof(argv[5]));

    cout << "Simulating " << num_gen << " generations" << endl
         << "Initial population: " << pool.getPopulation() << endl
         << "Max population: " << pool.getMaxPopulation() << endl
         << "Mutation rate: " << pool.getMutationRate() << endl
         << "Reproduction rate: " << pool.getReproductionRate() << endl
         << "Simulation length: " << pool.getSimulationLength() << endl;

    float best = 0.0f;

    for (int i = 0; i < num_gen; ++i)
    {
        pool.doGeneration();
        float newBest = pool.getBestFitness();
        cout << "+ Best fitness is " << newBest << endl << flush;
        pool.write(argv[2]);

        if (newBest > best)
        {
            best = newBest;
            pool.write(".best.file");
        }
    }

    pool.write(argv[2]);

    return 0;
}

