#include "Sim.h"
#include <cstdio>

extern "C" {
extern FILE *yyin;  // the file from which the scanner reads
int yylex();        // the scan function itsef
}

// The type of the value associated with each token
struct Token {
    int lineNum;
    int colNum;
    float value;
};

static Body::container *  _bodies;
static Joint::container * _joints;
static Exp::container *   _exps;

static bool parse_error;

extern "C" {

void
yyerror(char *mesg)
{
    cerr << "Parse error: " << mesg << endl;
    parse_error = true;
}

}

struct V2fproxy
{
    float x;
    float y;
};

#include "y.tab.c"
#include "lex.yy.c"

int
parseFile(const string &filename, Body::container &bodies, 
          Joint::container &joints, Exp::container &exps)
{
    bodies.clear();
    _bodies = &bodies;

    joints.clear();
    _joints = &joints;

    exps.clear();
    _exps = &exps;

    parse_error = false;

    yyin = fopen(filename.c_str(), "r");

    if (yyin == 0)
    {
        cerr << "<parseFile> Unable to open " << filename << endl;
        return -1;
    }

    yyparse();
    fclose(yyin);

    if (parse_error)
    {
        return -1;
    }

    // assign the first expressions to the active joints
    // these can be changed later
    unsigned int index = 0;
    for (Joint::container::iterator i = joints.begin(); 
         i != joints.end(); 
         ++i)
    {
        if (dynamic_cast<ConstantJoint *>(*i) != 0) continue;

        if (index >= exps.size())
        {
            cerr << "Parse error: not enough expressions" << endl;
            return -1;
        }

        (*i)->target_exp = exps[index];
        index += 1;
    }

    return 0;
}

