#ifndef _Sim_h_
#define _Sim_h_

#include <iostream>
#include <string>

using namespace std;

#ifndef _Body_h_
# include "Body.h"
#endif

#ifndef _Joint_h_
# include "Joint.h"
#endif

#ifndef _Exp_h_
# include "Exp.h"
#endif

class Sim
{
  public:

    Sim();
    ~Sim();

    int   parse(const string &filename);
    void  unparse(ostream &out) const;

    float getGroundLevel() const;
    void  doTimestep();
    void  reset();

    Body::container  bodies;
    Joint::container joints;
    Exp::container   exps;

    float getCurrentTime() const { return _currentTime; }
    float getTimestep() const { return timestep; }
    float getMaximumHeight() const { return _maxHeight; }
    bool  hadSelfCollision() const { return _selfCollided; }

    void  deleteAll();

  private:

    float             _currentTime;
    float             _maxHeight;
    bool              _selfCollided;

    void calcConstantForces(float dt);
    void predictFuturePositions(float dt);
    void calcCorrectionForces(float dt);
    void integrate(float dt);

    static const float timestep;
    static const float mouse_k;
    static const float mouse_b;
    static const float joint_k;
    static const float joint_b;
    static const float joint_ang_k;
    static const float joint_ang_b;
    static const float gravity;
    static const float ground_k;
    static const float ground_b; 
    static const float ground_level;
    static const float ground_friction;
};

#endif // _Sim_h_
