#ifndef _Vec2_h_
#define _Vec2_h_

#include <math.h>
#include <iostream>

class V2f
{
  public:

    //
    // Constructors
    // 
    V2f()                       { x = 0.0f; y = 0.0f; }
    V2f(float x, float y)       { this->x = x; this->y = y; }
    V2f(const V2f &v)           { x = v.x; y = v.y; }
    explicit V2f(float val)     { x = val; y = val; }

    ~V2f() {}

    //
    // Assignment and copy constructor
    //
    const V2f &
    operator = (const V2f &v)
    {
        x = v.x;
        y = v.y;
        return *this;
    }

    const V2f &
    operator = (float f)
    {
        x = f;
        y = f;
        return *this;
    }

    // 
    // Math operations
    //
    const V2f
    operator + (const V2f &v) const
    {
	return V2f(x + v.x, y + v.y);
    }
    const V2f
    operator - (const V2f &v) const
    {
	return V2f(x - v.x, y - v.y);
    }
    const V2f
    operator - () const
    {
	return V2f(-x, -y);
    }

    const V2f &
    operator += (const V2f &v)
    {
	x += v.x; y += v.y;
	return *this;
    }
    const V2f &
    operator -= (const V2f &v)
    {
	x -= v.x; y -= v.y;
	return *this;
    }

    //
    // scaling
    //
    const V2f &
    operator *= (float f)
    {
	x *= f; y *= f;
	return *this;
    }
    const V2f &
    operator /= (float f)
    {
	x /= f; y /= f;
	return *this;
    }

    const V2f
    operator * (float f) const
    {
	return V2f(x * f, y * f);
    }
    const V2f
    operator / (float f) const
    {
	return V2f(x / f, y / f);
    }

    //
    // dot product
    //
    float
    operator ^ (const V2f &v) const
    {
	return x * v.x + y * v.y;
    }

    //
    // signed area
    //
    float
    operator % (const V2f &v) const
    {
	return x * v.y - y * v.x;
    }

    //
    // magnitude
    //
    float
    magnitude() const
    {
	return (float) sqrt(x*x + y*y);
    }


    //
    // equality
    //
    bool
    operator == (const V2f &v) const
    {
        return (x == v.x) && (y == v.y);
    }

    bool
    operator != (const V2f &v) const
    {
        return (x != v.x) || (y != v.y);
    }

    //
    // X, Y values
    //
    float x;
    float y;
};

inline
istream &
operator >> (istream &in, V2f &v)
{
    if (!in.good()) return in;
   
    V2f tmp;

    if (!(in >> tmp.x))
    {
	// eof on first, stream at end
	if (!in.eof()) in.clear(ios::badbit);
	return in;
    }
    if (!(in >> tmp.y))
    {
	// eof on second, corrupt stream
	in.clear(ios::badbit);
	return in;
    }

    v = tmp;
    in.setstate(ios::goodbit);

    return in;
}

inline
ostream &
operator << (ostream &out, const V2f &v)
{
    out << "(" << v.x << ", " << v.y << ")";
    return out;
}

#endif
